This O'Reilly Webcast [Programming AVR Microcontrollers in C](https://www.youtube.com/watch?v=ERY7d7W-6nA) really helped me understand AVR programming and micro-controller in general.

# How to Read Datasheet ?

Datasheet is our Frenemy. For Attiny44 it is 286 pages long. It is 660 pages long for mega series chip.
It should be read as an encyclopedia, not as a novel.

First 2/3 pages are really good read. We will be referring it over and over again during programming. The important information contained in these pages are,

- Summary of all the peripherals and abilities.
- Power consumption, Max voltage.
- All the specs.
- Pin-out diagram, functions of all the different pins.

# First Page

<p align="center">
![1stpage][1stpage]
[1stpage]: http://localhost:3000/week09/1stpage.png ""
</p>

## RISC Architecture

[RISC vs CISC](https://cs.stanford.edu/people/eroberts/courses/soco/projects/risc/risccisc/) is great article to understand different microcontroller architecture. RISC came after CISC. The primary goal of CISC architecture is to complete a task in as few lines of assembly as possible.

```C
MULT 2:3, 5:2
```

RISC processors only use simple instructions that can be executed within one clock cycle.

```C
LOAD A, 2:3
LOAD B, 5:2
PROD A, B
STORE 2:3, A
```

- Disadvantages:
  More RAM is needed to store the assembly level instructions in RISC. The compiler must also perform more work to convert a high-level language statement into code of this form.
- Advantages:
  Each instruction requires only one clock cycle to execute. The entire program will execute in approximately the same amount of time as the multi-cycle command.

# Second Page

<p align="center">
![2ndpage][2ndpage]
[2ndpage]: http://localhost:3000/week09/2ndpage.png ""
</p>

<p align="center">
![attiny_pinout][attiny_pinout]
[attiny_pinout]: http://localhost:3000/week09/attiny_pinout.jpg ""
</p>

- RESET: Tell the AVR to enter programming mode.
- RXD, TXD:
- INT0:
- OC2B/INT1:
- T0:
- VCC:
- GND:
- XTAL1:
- XTAL2:
- OC0B/T1:
- OC0A/AIN0:
- AIN1:
- CLKO:
- ADC:
- SCL,SDA:
- AREF:
- AVCC:
- SCK: Serial Clock
- MISO: Master-in, Slave-out
- MOSI: Master-out, Slave-in
- OC2A:
- SS:
- OC1B:
- OC1A:

The next pages are to be read only when we need.  
First we need to get broad overview of the chapters and then skip over to **Register Description**. The important part required while programing is the register configuration, that is flipping bits of register.

# Microcontrollers

Microcontrollers are whole computer on a chip, but it's a very little computer. It has CPU (1-20 MHz), Dynamic memory (SRAM), Non-volatile memory (Flash ROM and EEPROM)

## Output

- Digital output: Apply 0V or 5V to any pin.
  - Digital communication: UART, SPI, I2C, USB
- Analog Output: Pulse-width Modulation for fake analog. (Arduino's analogWrite())

## Input

Read voltage levels applied to pins

- Digital input from pushbuttons and threshold sensors. Receiving digital
- Analog-to-Digital Conversion
- Input Modes
  - "Hi-Z": effectively disconnected from the circuit.
  - Internal pullup resistor: test if anything is connected.

The right strategy to approach microcontroller when doing multiple things at once is to offload tasks to on-board hardware so that the CPU stays free and just coordinating the tasks. That's why it is important to learn the internal hardware peripherals.

# The Toolchain

## Lifecycle of AVR Programming

- Write code in C
- Cross-compile for the chip
- Transfer the code to the chip
  - Hardware programmer to talk to the chip.
  - Software to run the programmer.

## Makefile

C is not a free standing language. We need a lot of stuff to compile for it. This is where **Make file** comes in.

- It keeps track of all the particular setup
- It runs all the right command line.
- It links in different batches of source file.
- It does **compile and flash** in one step.

# Peripherals and their Configuration

- Timers: for PWM,counting, timing, scheduling events etc. Using this well we can write an operating system.
- Interrupts: Using interrupts well is what separates a good microcontroller C programmer from C programmer. It lets us run a specific routine whenever an event happens which is useful in monitoring. It pauses the running code and responds to event promptly.
- Serial I/O : built-in hardware for USART, SPI, I2C available in AVR Atmega chips.

## Registers

- Registers ("Special function registers") are fixed memory locations with side-effects.
- Read and write just like normal variable.
- Each register byte is bits - comparable to 8 switches lined up.
- Each switch has a side-effect, depends on which register, which bit
- PORTB =0b00000010; This sets the PORTB register's value to flip the number 1 bit (Count from right to left starting from zero) which turns on PB1.

- Input/Output
  - I/O is also a hardware peripheral.
    - We use special register DDR (Data direction Registers) to control the pin's data direction.
    - DDRB = 0b00000010 sets pin one in PORTB(PB1) into output mode and rest in Input mode.

# Bit Twiddling

## bit-shifting

- Left shift(<<): This improves code readability

```C
(1 << PB1) -> 0b00000010;
```

## bitwise logical operations

- OR (|): Set bit with OR

```C
PORTB |= (1 << PB1);
```

- XOR (^): Toggling bits

```C
PORTB ^= (1 << PB1);
```

## Clearing bits with NOT and AND

To turn off the bit corresponding to PB1 without affecting other pins.

```C
// bit-masking.
PORTB &= ~(1 << PB1);
```

## Shortcuts

```C
// set port direction for output
#define output(directions,pin) (directions |= pin)
// get Bit Value by Bit-Shifting.
#define BV(pin) (1 << pin)
// set port pin using OR
#define setBit(port,pin) (port |= BV(pin) ) //digitalWrite(pin,HIGH)
// clear port pin using AND
#define clearBit(port,pin) (port &= ~BV(pin)) //digitalWrite(pin,Low)
// toggle port pin using XOR
#define toggleBit(port,pin) (port ^= BV(pin))
```

# Arduino programming vs Embedded C programming

## Why C Programming?

- **Speed**: C code runs much faster
  - digitalWrite takes about 40 cycles in Arduino Code.
  - C code takes one cycle
- **Flexibility**: Make the hardware peripherals do what we want and when want them to.
- **Responsiveness**: Using interrupt system well we can do many things at once, respond instantly
  - Interrupts are basically events that require immediate attention by the microcontroller. When an interrupt event occurs the microcontroller pause its current task and attend to the interrupt by executing an Interrupt Service Routine (ISR) at the end of the ISR the microcontroller returns to the task it had pause and continue its normal operations.
- **Portability**: C is available for every CPU
  - Code will work on all AVRs

## Similarity b/w Arduino Programming and AVR Programming

- Control external devices by connecting them to AVR pins.
- Communicate with desktop PC over serial.
- PWM, ADC, initialization/ event loop structure.

## Difference

- Different toolchain to program chip.
- Configure internal hardware peripherals.
- microcontroller-C programming idioms.
